'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  /* ==========================================================================
   * Utilities
   * ======================================================================= */

  var transformOnType = function transformOnType(transforms) {
    return function (node) {
      var transformer = transforms[node.type];
      if (transformer) {
        return transformer(node);
      }
      throw new Error(node.type + ' could not be transformed');
    };
  };

  /* ==========================================================================
   * Node Transformers
   * ======================================================================= */

  var JSXIdentifier = function JSXIdentifier(node) {
    return t.stringLiteral(node.name);
  };

  var JSXNamespacedName = function JSXNamespacedName(node) {
    return t.stringLiteral(node.namespace.name + ':' + node.name.name);
  };

  var _JSXMemberExpression = transformOnType({
    JSXIdentifier: function JSXIdentifier(node) {
      return t.identifier(node.name);
    },
    JSXMemberExpression: function JSXMemberExpression(node) {
      return t.memberExpression(_JSXMemberExpression(node.object), _JSXMemberExpression(node.property));
    }
  });

  var JSXElementName = transformOnType({ JSXIdentifier: JSXIdentifier, JSXNamespacedName: JSXNamespacedName, JSXMemberExpression: _JSXMemberExpression });

  var JSXExpressionContainer = function JSXExpressionContainer(node) {
    return node.expression;
  };

  var JSXAttributeName = JSXElementName;

  var JSXAttributeValue = transformOnType({
    StringLiteral: function StringLiteral(node) {
      return node;
    },
    JSXExpressionContainer: JSXExpressionContainer
  });

  var JSXAttributes = function JSXAttributes(state) {
    return function (nodes) {
      var object = [];
      var objects = [];

      nodes.forEach(function (node) {
        switch (node.type) {
          case 'JSXAttribute':
            {
              if (!object) {
                object = [];
              }

              var attributeName = JSXAttributeName(node.name);

              var objectKey = _esutils2.default.keyword.isIdentifierNameES6(attributeName.value) ? t.identifier(attributeName.value) : attributeName;

              object.push(t.objectProperty(objectKey, JSXAttributeValue(node.value)));
              break;
            }
          case 'JSXSpreadAttribute':
            {
              if (object) {
                objects.push(t.objectExpression(object));
                object = null;
              }

              objects.push(node.argument);
              break;
            }
          default:
            throw new Error(node.type + ' cannot be used as a JSX attribute');
        }
      });

      if (object && object.length > 0) {
        objects.push(t.objectExpression(object));
      }

      if (objects.length === 0) {
        return t.objectExpression([]);
      } else if (objects.length === 1) {
        return objects[0];
      }

      return t.callExpression(state.addHelper('extends'), objects);
    };
  };

  var JSXText = function JSXText(node) {
    return t.stringLiteral(node.value.replace(/\s+/g, ' '));
  };

  var JSXChild = function JSXChild(state) {
    return transformOnType({ JSXText: JSXText, JSXElement: _JSXElement(state), JSXExpressionContainer: JSXExpressionContainer });
  };

  var JSXChildren = function JSXChildren(state) {
    return function (nodes) {
      return t.arrayExpression(nodes.map(JSXChild(state)));
    };
  };

  var _JSXElement = function _JSXElement(state) {
    return function (node) {
      return state.get('jsxObjectMapper')(t.objectExpression([t.objectProperty(t.identifier(nameProperty), JSXElementName(node.openingElement.name)), t.objectProperty(t.identifier(attributesProperty), JSXAttributes(state)(node.openingElement.attributes)), t.objectProperty(t.identifier(childrenProperty), node.closingElement ? JSXChildren(state)(node.children) : t.nullLiteral())]));
    };
  };

  /* ==========================================================================
   * Misc Functions
   * ======================================================================= */

  var initJSX = function initJSX(path, state) {
    var _state$opts = state.opts;
    var _state$opts$useNew = _state$opts.useNew;
    var useNew = _state$opts$useNew === undefined ? false : _state$opts$useNew;
    var constructorModule = _state$opts.module;
    var constructorFunction = _state$opts.function;

    var executeExpression = useNew ? t.newExpression : t.callExpression;
    var valueWrapper = function valueWrapper(expression) {
      return function (value) {
        return executeExpression(expression, [value]);
      };
    };

    if (constructorModule) {
      var moduleName = path.scope.generateUidIdentifier(constructorFunction || useNew ? 'JSXNode' : 'jsx');
      state.set('jsxObjectMapper', valueWrapper(moduleName));

      var importDeclaration = t.importDeclaration([t.importDefaultSpecifier(moduleName)], t.stringLiteral(constructorModule));

      path.findParent(function (p) {
        return p.isProgram();
      }).unshiftContainer('body', importDeclaration);
    } else if (constructorFunction) {
      var expression = constructorFunction.split('.').map((0, _ary2.default)(t.identifier, 1)).reduce((0, _ary2.default)(t.memberExpression, 2));
      state.set('jsxObjectMapper', valueWrapper(expression));
    } else {
      state.set('jsxObjectMapper', _identity2.default);
    }
  };

  /* ==========================================================================
   * Plugin
   * ======================================================================= */

  return {
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      JSXElement: function JSXElement(path, state) {
        if (!state.get('jsxInit')) {
          initJSX(path, state);
          state.set('jsxInit', true);
        }

        // Replace JSX with an object.
        path.replaceWith(_JSXElement(state)(path.node));
      }
    }
  };
};

var _identity = require('lodash/utility/identity');

var _identity2 = _interopRequireDefault(_identity);

var _ary = require('lodash/function/ary');

var _ary2 = _interopRequireDefault(_ary);

var _esutils = require('esutils');

var _esutils2 = _interopRequireDefault(_esutils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nameProperty = 'elementName';
var attributesProperty = 'attributes';
var childrenProperty = 'children';